/***************************************************************************
                           main.cpp  -  description
                             -------------------
    begin                : Mon Mar 4 2002
    copyright            : (C) 2002 by Jerome PANSANEL
    email                : pansanel@pansanel.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation; either version 2.1 of the  *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 ***************************************************************************/

#include "molecule.h"
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <fstream.h>
#define DEFAULT_TABLE "table.txt"


/*String returned, when the false argument is called with the programm.*/
void error(void)
{
      cerr << "mwcalc: missing some arguments\n";
      cerr << "Try \'mwcalc -h\' for more information\n";
      exit(0);
}

int main(int argc, char * argv[])
{
  if (argc <= 1)
  {
    error();
  }

  long int state = 0;
  char * definition_file = new char[16];
  *definition_file = '\0';
  char * formula = new char[65536];
  *formula = '\0';
  molecule compound;

  while ((argc > 1) && (argv[1][0] == '-'))
  {
    switch (argv[1][1])
    {
      case 'h':
      {
        cout << "Usage: mwcalc [OPTION]... FORMULA\n"
             << "Calculate the molecular weight of FORMULA\n"
	     << endl
	     << "  -f FILE      open a FILE containing the formula\n"
             << "  -h           display this help and exit\n"
             << "  -l FILE      open a FILE containing the element definition\n"
             << "  -p           display the parsed formula\n"
             << "  -v           verbose; display a simple text about the formula\n"
	     << "  -V           display version information and exit\n"
	     << endl
	     << "Report bugs to <pansanel@pansanel.net>.\n";
        exit(0);
      }
      case 'l':
      {
	if (argc <= 2)
	{
	  error();
	}
        --argc;
        ++argv;
        strncpy(definition_file, argv[1], strlen(argv[1]));
        break;
      }
      case 'a':
      {
	      state |= (1 << 1);
        break;
      }

      case 'f':
      {
	if (argc <= 2)
	{
	  error();
	}
        argc--;
        argv++;
	ifstream elem;
	elem.open(argv[1], ios::in);
	if(!elem.is_open())
	{
		cerr << "mwcalc: " << argv[1] << ": No such file or directory" << endl;
		exit(0);
	}
        elem.getline(formula, 65535, '\n');
	elem.close();
	break;
      }
      
      case 'p':
      {
	state |= (1 << 0);
        break;
      }

      case 'v':
      {
	state |= (1 << 2);
        break;
      }

      case 'V':
      {
        cout << "mwcalc, version 1.2, 28 july 2002" << endl;
        exit(0);
      }

      default:
      {
        error();
      }
    }
    --argc;
    ++argv;
  }
  if (argc <= 1 && *formula == '\0')
  {
	  error();
  }
  if (argc == 2 && *formula == '\0')
  {
	  strcpy(formula, argv[1]);
  }
  if (*definition_file == '\0')
  {
	  strncpy(definition_file, DEFAULT_TABLE, strlen(DEFAULT_TABLE));
  }
  molecule query;
  query.parse(formula);
  query.set_atom_mass(definition_file);
  query.calcul_weight();
  if ((state & (1 << 2)) == (1 << 2))
  {
	  cout << "The molecule weight : ";
  }
  cout << query.get_weight();
  if ((state & (1 << 2)) == (1 << 2))
  {
	  cout << " g/mol";
  }
  if ((state & (1 << 0)) == (1 << 0))
  {
	  cout << endl;
	  query.get_parsed_formula();
  }
  if ((state & (1 << 1)) == (1 << 1))
  {
	  cout << endl;
	  query.get_elementar_analysis();
  }
  cout << endl;
  return(0);
}
